unit UbView;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	ExtCtrls;

const
	LAST_VER = '0.38';

	MARG = 4; //margine testo nelle celle

type
	TPicShowing = (pcNone, pcSolid, pcCenter, pcScaled, pcRepeated);

	TUbImage = class( TImage )
 	protected
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
	public
		constructor Create(AOwner: TComponent); override;
		procedure Paint; override; //promosso
  end;


	TUbView = class(TGraphicControl)
	private
		FPicture: TPicture;
		FColor: TColor;
		FTransparent: Boolean;
		procedure PictureChanged(Sender: TObject);
		procedure SetColor(Value: TColor);
		procedure SetPicture(Value: TPicture);
		procedure SetTransparent(Value: Boolean);
		procedure SetVersion(NewVersion: string);
 	protected
		FPictureShowing: TPicShowing;
		function GetVersion: string;
		procedure SetPictureShowing(NewStatus:TPicShowing);
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
	public
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;
		procedure Paint; override; //promosso
	published
		property Color: TColor read FColor write SetColor;
		property Picture: TPicture read FPicture write SetPicture;
		property PictureShowing:TPicShowing read FPictureShowing write SetPictureShowing;
		property Transparent: Boolean read FTransparent write SetTransparent default False;
		property Version: string read GetVersion write SetVersion stored false;
// inherited
		property Align;
    property DragKind;
    property DragCursor;
    property DragMode;
    property PopupMenu;
		property Visible;
    property OnCanResize;
    property OnConstrainedResize;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
	end;


TUbRowType = (rtBody, rtAltBody, rtHeader, rtAltHeader);

TUbRow = class(TCollectionItem)
	private
		FStrings: TStringList;
		FType: TUbRowType;
		procedure SetRowType(AValue: TUbRowType);
	protected
	public
		constructor Create(Collection: TCollection); override;
		procedure Assign(Pt: TPersistent); override;
		destructor Destroy; override;
		procedure Put(index: integer;const s: string);
		function Get(index: integer): string;
		procedure Clear;
		property Fields[Num: Integer]: string read Get write Put;
	published
		property RowType: TUbRowType read FType write SetRowType;
		property FieldsList: TStringList read FStrings write FStrings;
	end;


TUbColumn = class(TCollectionItem)
	private
		FAlignment: TAlignment;
		FAltColor: TColor;
		FBorderHorWidth: integer;
		FBorderVerWidth: integer;
		FColor: TColor;
		FFont: TFont;
		FHeader: string;
		FParentColor: boolean;
		FParentFont: boolean;
		FWidth: integer;
		procedure SetFont(Value: TFont);
	public
		constructor Create(Collection: TCollection); override;
		destructor Destroy; override;
		procedure Assign(Pt: TPersistent); override;
	published
		property Alignment: TAlignment read FAlignment write FAlignment;
		property AlternateColor: TColor read FAltColor write FAltColor;
		property BorderHorWidth: integer read FBorderHorWidth write FBorderHorWidth;
		property BorderVerWidth: integer read FBorderVerWidth write FBorderVerWidth;
		property Color: TColor read FColor write FColor;
		property Font: TFont read FFont write SetFont;
		property Header: string read FHeader write FHeader;
		property ParentColor: boolean read FParentColor write FParentColor;
		property ParentFont: boolean read FParentFont write FParentFont;
		property Width: integer read FWidth write FWidth;
	end;


TUbViewGrid = class;


TUbRows = class(TCollection)
	private
		FUbViewGrid: TUbViewGrid; //owner
		function GetItem(Index: Integer): TUbRow;
		procedure SetItem(Index: Integer; Value: TUbRow);
	protected
		function GetOwner: TPersistent; override;
		procedure Update(Item: TCollectionItem); override;
	public
		constructor Create(UbViewGrid: TUbViewGrid);
		function Add: TUbRow;
		procedure LoadFromFile(const Filename: string);
		procedure LoadFromStream(S: TStream);
		procedure SaveToFile(const Filename: string);
		procedure SaveToStream(S: TStream);
		property Items[Index: Integer]: TUbRow read GetItem write SetItem; default;
	end;


TUbColumns = class(TCollection)
	private
		FUbViewGrid: TUbViewGrid; //owner
		function GetItem(Index: Integer): TUbColumn;
		procedure SetItem(Index: Integer; Value: TUbColumn);
	protected
		function GetOwner: TPersistent; override;
		procedure Update(Item: TCollectionItem); override;
	public
		constructor Create(UbViewGrid: TUbViewGrid);
		function Add: TUbColumn;
		property Items[Index: Integer]: TUbColumn read GetItem write SetItem; default;
	end;



TUbViewGrid = class(TUbView)
	private
		F3dGrid: boolean;
		FBorderCol: TColor;
		FColsStyle: TUbColumns;
		FCurrTime: integer;
		FFirstVisibleRow: integer;
		FFont: TFont;
		FLastVisibleRow : integer;
		FOnRepeat: TNotifyEvent;
		FOnChangingPage: TNotifyEvent;
		FOnChangedPage: TNotifyEvent;
		FRowHeight: integer;
		FRowsNumber: integer;
		FRows: TUbRows;
		FShowColHea: boolean;
		FTimer: TTimer;
		FTimesToRepeat: integer;
		FWaitForPage: integer;
		procedure FixLastVisibleRow;
		procedure OnTimer(Sender: TObject);
		procedure SetFont(Value: TFont);
		procedure Set3dGrid(Value: boolean);
		procedure SetRowHeight(Value: integer);
		procedure SetRowsNumber(Value: integer);
		procedure SetTimesToRepeat(Value: integer);
		procedure SetWaitForPage(Value: integer);
	protected
		FHeaderStyle: TUbColumn;
		procedure DrawRow(ACanvas: TCanvas; Row: TUbRow; Y: integer); virtual;
	public
		procedure Paint; override;
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;
		procedure AddBodyRow(AStrings: TStrings; Alternate: boolean);
		procedure AddHeaderRow(const Text: string; Alternate: boolean);
		procedure DeleteRow(RowNum : integer);
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property FirstVisibleRow: integer read FFirstVisibleRow;
	published
		property BorderColor: TColor read FBorderCol write FBorderCol;
		property ColsStyle:TUbColumns read FColsStyle write FColsStyle;
		property Grid3D: boolean read F3dGrid write Set3dGrid;
		property Font:TFont read FFont write SetFont;
		property HeaderStyle: TUbColumn read FHeaderStyle write FHeaderStyle;
		property OnRepeat: TNotifyEvent read FOnRepeat write FOnRepeat;
		property OnChangingPage: TNotifyEvent read FOnChangingPage write FOnChangingPage;
		property OnChangedPage: TNotifyEvent read FOnChangedPage write FOnChangedPage;
		property RowHeight: integer read FRowHeight write SetRowHeight;
		property RowsNumber: integer read FRowsNumber write SetRowsNumber;
		property Rows:TUbRows read FRows write FRows;
		property ShowColumnHeaders: boolean read FShowColHea write FShowColHea;
		property TimesToRepeat: integer read FTimesToRepeat write SetTimesToRepeat;
		property WaitForPage: integer read FWaitForPage write SetWaitForPage;
	end;


procedure Register;

implementation

constructor TUbView.Create(AOwner: TComponent);
	begin
	inherited Create(AOwner);
	ControlStyle := ControlStyle + [csReplicatable] - [csOpaque];
	FPicture := TPicture.Create;
	FPicture.OnChange := PictureChanged;
	Height := 105;
	Width := 105;
	Fcolor := clBtnFace;
	end;


destructor TUbView.Destroy;
	begin
	FPicture.Free;
	inherited Destroy;
	end;

function TUbView.GetVersion: string;
	begin
	result := LAST_VER;
	end;

procedure TUbView.SetVersion(NewVersion: string);
	begin
	//non faccio un tubo
	end;

procedure TUbView.SetPictureShowing(NewStatus:TPicShowing);
	begin
	FPictureShowing := NewStatus;
	Invalidate;
	end;


procedure TUbView.SetPicture(Value: TPicture);
	begin
	FPicture.Assign(Value);
	invalidate;
	end;


procedure TUbView.PictureChanged(Sender: TObject);
	begin
	invalidate;
	end;

procedure TUbView.Paint;
	var
		hw, hh: integer;
	begin
	if csDesigning in ComponentState then
		with inherited Canvas do
			begin
			Pen.Style := psDash;
			Brush.Style := bsClear;
			Rectangle(0, 0, Width, Height);
			end;

	with inherited Canvas do
		begin
		case FPictureShowing of
			pcNone:
				begin
				end;
			pcSolid:
				begin
				Pen.Style := psSolid;
				Pen.Color := FColor;
				Brush.Style := bsSolid;
				Brush.Color := FColor;
				Rectangle(0, 0, Width, Height);
				end;
			pcCenter:
				begin
				if (Picture.Width = 0) or (Picture.Height = 0) then
					exit;
				hw := (Width - Picture.Width) div 2;
				hh := (Height - Picture.Height) div 2;
				Draw(hw, hh, Picture.Graphic);
				end;
			pcScaled:
				begin
				if (Picture.Width = 0) or (Picture.Height = 0) then
					exit;
				StretchDraw(Rect(0, 0, Width, Height), Picture.Graphic);
				end;
			pcRepeated:
				begin
				hh := 0;
				if (Picture.Width = 0) or (Picture.Height = 0) then
					exit;
				while hh < Height do
					begin
					hw := 0;
					while hw < Width do
						begin
						Draw(hw, hh, Picture.Graphic);
						hw := hw + Picture.Width;
						end;
					hh := hh + Picture.Height;
					end;
				end;
			end;
		end;
	end;

procedure TubView.SetColor(Value: TColor);
	begin
	FColor := Value;
	if FPictureShowing = pcSolid then
		invalidate;
	end;

procedure TubView.SetTransparent(Value: Boolean);
	begin
	if Value <> FTransparent then
		begin
		FTransparent := Value;
		if Value then
      begin
      Picture.Bitmap.Transparent := true;
      Picture.Bitmap.TransparentMode := tmAuto;
      end
    else
      Picture.Bitmap.Transparent := false;
		invalidate;
		end;
	end;

procedure TubView.WMEraseBkgnd(var Message: TWMEraseBkgnd);
  begin
  if Transparent then
    inherited;
  end;

///////////////////////////////
///  TUbRow
///////////////////////////////

constructor TUbRow.Create(Collection: TCollection);
	begin
	inherited;
	FStrings := TStringList.Create;
	FType := rtBody;
	end;


procedure TUbRow.Assign(Pt: TPersistent);
	begin
	inherited;
	if Pt is TUbRow then
		begin
		FStrings.Assign(TUbRow(Pt).FStrings);
		FType := TUbRow(Pt).FType;
		end
	else
		// raise an exception
		inherited Assign (pt);
	end;


destructor TUbRow.Destroy;
	begin
	FStrings.Free;
	inherited;
	end;

procedure TUbRow.SetRowType(AValue: TUbRowType);
	begin
	FType := AValue;
	end;

procedure TUbRow.Put(index: integer; const s: string);
	var
		k: integer;
	begin
	if index < 1 then
		exit;
	if index > FStrings.Count then
		for k := FStrings.Count to index do
			FStrings.Add('');
	FStrings.Insert(index - 1, s);
	end;


function TUbRow.Get(index: integer): string;
	begin
	if (index < 1) or (index > FStrings.Count) then
		result := ''
	else
		result := FStrings[index - 1];
	end;

procedure TUbRow.Clear;
	begin
	FStrings.Clear;
	end;

///////////////////////////////
///  TUbColumn
///////////////////////////////

constructor TUbColumn.Create(Collection: TCollection);
	begin
	inherited;
	FFont := TFont.Create;
	FAlignment := taLeftJustify;
	FBorderHorWidth := 0;
	FBorderVerWidth := 0;
	FColor := clWhite;
	FAltColor := clLime;
	FParentColor := true;
	FParentFont := true;
	FWidth := 80;
	end;

destructor TUbColumn.Destroy;
	begin
	FFont.Free;
	inherited;
	end;

procedure TUbColumn.Assign (Pt: TPersistent);
	begin
	if Pt is TUbColumn then
		begin
		FFont.Assign(TUbColumn(pt).Font);
		FAlignment := TUbColumn(pt).Alignment;
		FAltColor := TUbColumn(pt).AlternateColor;
		FBorderHorWidth := TUbColumn(pt).BorderHorWidth;
		FBorderVerWidth := TUbColumn(pt).BorderVerWidth;
		FColor := TUbColumn(pt).Color;
		FHeader := TUbColumn(pt).Header;
		FParentColor := TUbColumn(pt).ParentColor;
		FParentFont := TUbColumn(pt).ParentFont;
		FWidth := TUbColumn(pt).Width;
		end
	else
		// raise an exception
		inherited Assign (pt);
	end;

procedure TUbColumn.SetFont(Value: TFont);
	begin
	FFont.Assign(Value);
	end;


///////////////////////////////
///  TUbColumns
///////////////////////////////

constructor TUbColumns.Create(UbViewGrid: TUbViewGrid);
begin
	inherited Create(TUbColumn);
	FUbViewGrid := UbViewGrid;
end;

function TUbColumns.Add: TUbColumn;
begin
	Result := TUbColumn(inherited Add);
end;

function TUbColumns.GetItem(Index: Integer): TUbColumn;
begin
	Result := TUbColumn(inherited GetItem(Index));
end;

function TUbColumns.GetOwner: TPersistent;
begin
	Result := FUbViewGrid;
end;

procedure TUbColumns.SetItem(Index: Integer; Value: TUbColumn);
begin
	inherited SetItem(Index, Value);
end;

procedure TUbColumns.Update(Item: TCollectionItem);
begin
FUbViewGrid.invalidate;
end;



///////////////////////////////
///  TUbRows
///////////////////////////////

constructor TUbRows.Create(UbViewGrid: TUbViewGrid);
	begin
	inherited Create(TUbRow);
	FUbViewGrid := UbViewGrid;
	end;

function TUbRows.Add: TUbRow;
	begin
	Result := TUbRow(inherited Add);
	end;

function TUbRows.GetItem(Index: Integer): TUbRow;
	begin
	Result := TUbRow(inherited GetItem(Index));
	end;

function TUbRows.GetOwner: TPersistent;
	begin
	Result := FUbViewGrid;
	end;

procedure TUbRows.SetItem(Index: Integer; Value: TUbRow);
	begin
	inherited SetItem(Index, Value);
	end;

procedure TUbRows.Update(Item: TCollectionItem);
	begin
	FUbViewGrid.invalidate;
	end;


type
	TUbRowsWrapper = class(TComponent)
	private
		FColumns: TUbRows;
	published
		property Rows: TUbRows read FColumns write FColumns;
	end;

procedure TUbRows.LoadFromFile(const Filename: string);
	var
		S: TFileStream;
	begin
	S := TFileStream.Create(Filename, fmOpenRead);
		try
		LoadFromStream(S);
		finally
		S.Free;
		end;
	end;


procedure TUbRows.LoadFromStream(S: TStream);
	var
		Wrapper: TUbRowsWrapper;
	begin
	Wrapper := TUbRowsWrapper.Create(nil);
		try
		Wrapper.Rows := FUbViewGrid.FRows.Create(FUbViewGrid); //se ne crea una copia
		S.ReadComponent(Wrapper);
		Assign(Wrapper.Rows);
		finally
		Wrapper.Rows.Free;
		Wrapper.Free;
		end;
	end;

procedure TUbRows.SaveToFile(const Filename: string);
	var
		S: TStream;
	begin
	S := TFileStream.Create(Filename, fmCreate);
		try
		SaveToStream(S);
		finally
		S.Free;
		end;
	end;

procedure TUbRows.SaveToStream(S: TStream);
	var
		Wrapper: TUbRowsWrapper;
	begin
	Wrapper := TUbRowsWrapper.Create(nil);
		try
		Wrapper.Rows := Self;
		S.WriteComponent(Wrapper);
		finally
		Wrapper.Free;
		end;
	end;


///////////////////////////////
///  TUbViewGrid
///////////////////////////////


constructor TUbViewGrid.Create(AOwner: TComponent);
	begin
	inherited;
	FHeaderStyle := TUbColumn.Create(nil);
	FFont := TFont.Create;
	FRows := TUbRows.Create(self);
	FColsStyle := TUbColumns.Create(self);
	FRowHeight := - FFont.Height + 4;
	FBorderCol := clBlack;
	FTimesToRepeat	:= 0;
	FWaitForPage	:= 0;
	FFirstVisibleRow := 0;
	FixLastVisibleRow;
	FCurrTime := 0;
	FOnRepeat:= nil;
	if not (csDesigning in ComponentState) then
		begin //niente timer a Design Time
		FTimer := TTimer.Create(self);
		FTimer.Enabled := false;
		FTimer.OnTimer := OnTimer;
		end;
	end;

destructor TUbViewGrid.Destroy;
	begin
	FHeaderStyle.Free;
	FFont.Free;
	FRows.Clear;
	FRows.Free;
	FColsStyle.Clear;
	FColsStyle.Free;
	inherited;
	end;

procedure TUbViewGrid.SetFont(Value: TFont);
	begin
	FFont.Assign(Value);
	if FRowHeight < (4 - FFont.Height) then
		FRowHeight := (4 - FFont.Height);
	Invalidate;
	end;

procedure TUbViewGrid.Set3dGrid(Value: boolean);
	begin
	F3dGrid := Value;
	invalidate;
	end;

procedure TUbViewGrid.SetRowHeight(Value: integer);
	begin
	if Value < 1 then
		exit;
	FRowHeight := Value;
	FRowsNumber := Height div Value;
	invalidate;
	end;

procedure TUbViewGrid.SetRowsNumber(Value: integer);
	begin
	if Value < 1 then
		exit;
	FRowsNumber := Value;
	FRowHeight := Height div Value;
	invalidate;
	end;

procedure TUbViewGrid.SetTimesToRepeat(Value: integer);
	begin
	FFirstVisibleRow := 0;
	FixLastVisibleRow;
	FTimesToRepeat := Value;
	if Assigned(FTimer) then
		begin
		if (Value > 0) and (WaitForPage > 0) then
			begin
			FTimer.Interval := WaitForPage * 1000; //in secondi
			FTimer.Enabled := true;
			FCurrTime := TimesToRepeat;
			end
		else
			begin
			FTimer.Enabled := false;
			FCurrTime := 0;
			end
		end;
	end;

procedure TUbViewGrid.OnTimer(Sender: TObject);
	begin
	if FFirstVisibleRow + RowsNumber > Rows.Count then
		begin
		FFirstVisibleRow := 0;
		FixLastVisibleRow;
		dec(FCurrTime);
		if FCurrTime < 0 then
			begin
			if Assigned(OnRepeat) then
				OnRepeat(self);
			FCurrTime := TimesToRepeat;
			end;
		end
	else
		begin
		FFirstVisibleRow := FLastVisibleRow + 1;
		FixLastVisibleRow;
		end;
  if Assigned(OnChangingPage) then
		OnChangingPage(self);
  Paint;
  if Assigned(OnChangedPage) then
		OnChangedPage(self);
	end;

procedure TUbViewGrid.SetWaitForPage(Value: integer);
	begin
	FWaitForPage := Value;
	FFirstVisibleRow := 0;
	FixLastVisibleRow;
	if Assigned(FTimer) then
		begin
		if (Value > 0) and (TimesToRepeat > 0) then
			begin
			FTimer.Interval := Value * 1000; //in secondi
			FTimer.Enabled := true;
			FCurrTime := TimesToRepeat;
			end
		else
			begin
			FTimer.Enabled := false;
			FCurrTime := 0;
			end
		end;
	end;

procedure TUbViewGrid.Paint;
	var
		i, y: integer;
		tmpCanvas: TCanvas;
		HeaderRow : TUbRow;
	begin

	inherited;

	if csDesigning in ComponentState then
		exit;
	Canvas.Font := FFont;
	if FRows.Count > 0 then
		begin
		tmpCanvas := Canvas;
		y := 0;
		if ShowColumnHeaders then
			begin
			HeaderRow := TUbRow.Create(nil);
				try
				HeaderRow.SetRowType(rtAltHeader);
				for i := 0 to FColsStyle.Count - 1 do
					HeaderRow.FStrings.Add(FColsStyle.Items[i].FHeader);
				DrawRow(tmpCanvas, HeaderRow, y);
				y := y + FRowHeight;
				finally
				HeaderRow.Free;
				end;
			end;
		for i := FFirstVisibleRow to FLastVisibleRow do
			begin
			if y > Height then
				break; //fuori dai limiti
			DrawRow(tmpCanvas, FRows.Items[i], y);
			y := y + FRowHeight;
			end;
		end;
	end;



procedure TUbViewGrid.DrawRow(ACanvas: TCanvas; Row: TUbRow; Y: integer);
	var
		x, k, cw : integer;
		s : string;
		TexExt: TSize;

	function DrawCell(ACol: TUbColumn; x, y, w: integer; AText: string; NormColor: boolean): integer; //ritorna la larghezza
		var
			tx, ty, h: integer;
		begin
		h := FRowHeight;
		with ACol do
			begin
      if w = 0 then //default quella della colonna
  			w := Width;
			if ParentColor then
				Acanvas.Brush.Style := bsClear
			else
				begin
				Acanvas.Brush.Style := bsSolid;
				if NormColor then
					Acanvas.Brush.Color := FColor
				else
					Acanvas.Brush.Color := FAltColor
				end;

			if ParentFont then
				ACanvas.Font := self.FFont
			else
				ACanvas.Font := FFont;

			TexExt:= ACanvas.TextExtent(AText);

			case FAlignment of
				taLeftJustify :
					begin
					tx := x + MARG;
					end;
				taRightJustify :
					begin
					tx := x + w - (TexExt.cx + MARG);
					end;
				else //taCenter
					begin
					tx := x + (w - TexExt.cx) div 2;
					end;
				end;

			ty := y + (h - TexExt.cy) div 2;

			ACanvas.TextRect(Rect(x, y, x + w, y + h), tx, ty, AText);

			if FBorderHorWidth > 0 then
				if F3dGrid then
					begin
					Acanvas.Pen.Style := psSolid;
					Acanvas.Pen.Color := clBtnShadow;
					ACanvas.MoveTo(x, y + h - 1);
					ACanvas.LineTo(x + w - 1, y + h - 1);
					Acanvas.Pen.Color := clBtnHighlight;
					ACanvas.MoveTo(x, y);
					ACanvas.LineTo(x + w - 1, y);
					end
				else
					begin
					Acanvas.Pen.Style := psClear;
					Acanvas.Brush.Style := bsSolid;
					Acanvas.Brush.Color := FBorderCol;
					ACanvas.FillRect(Rect(x, y + h - FBorderHorWidth, x + w, y + h));
					end;

			if FBorderVerWidth > 0 then
				if F3dGrid then
					begin
					Acanvas.Pen.Style := psSolid;
					Acanvas.Pen.Color := clBtnShadow;
					ACanvas.MoveTo(x + w - 1, y);
					ACanvas.LineTo(x + w - 1, y + h - 1);
					Acanvas.Pen.Color := clBtnHighlight;
					ACanvas.MoveTo(x, y);
					ACanvas.LineTo(x, y + h - 1);
					end
				else
					begin
					Acanvas.Pen.Style := psClear;
					Acanvas.Brush.Style := bsSolid;
					Acanvas.Brush.Color := FBorderCol;
					ACanvas.FillRect(Rect(x + w - FBorderVerWidth, y, x + w, y + h));
					end

			end;

		result := w;
		end;


	begin
	with Row do
		begin
		x := 0;
		case FType of
			rtBody, rtAltBody:
				begin
				if FColsStyle.Count < 1 then
					exit;
				for k := 0 to FColsStyle.Count - 1 do
					begin
					s := Get(k + 1); //testo
					x := x + DrawCell(TUbColumn(FColsStyle.Items[k]), x, y, 0, s, (FType = rtBody));
					if x > Width then
						break; //fuori dai limiti
					end;
				end;
			rtHeader:
				begin
				s := Get(1);
				DrawCell(FHeaderStyle, x, y, 0, s, true);
				end;
			 rtAltHeader: //intestazioni colonne
				begin
				if FColsStyle.Count < 1 then
					exit;
				for k := 0 to FColsStyle.Count - 1 do
					begin
					s := Get(k + 1);
          cw := TUbColumn(FColsStyle.Items[k]).Width;
					x := x + DrawCell(FHeaderStyle, x, y, cw, s, false);
					if x > Width then
						break; //fuori dai limiti
					end;
				end;
			end;

		end;
	end;


procedure TUbViewGrid.AddBodyRow(AStrings: TStrings; Alternate: boolean);
	var
		tmpRow: TUbRow;
	begin
	tmpRow := TUbRow(FRows.Add);
	with tmpRow do
		begin
		FStrings.Assign(AStrings);
		if Alternate then
			FType := rtAltBody
		else
			FType := rtBody;
		end;
	invalidate;
	end;

procedure TUbViewGrid.AddHeaderRow(const Text: string; Alternate: boolean);
	var
		tmpRow: TUbRow;
	begin
	tmpRow := TUbRow(FRows.Add);
	with tmpRow do
		begin
		FStrings.add(Text);
		if Alternate then
			FType := rtAltHeader
		else
			FType := rtHeader;
		end;
	invalidate;
	end;

procedure TUbViewGrid.DeleteRow(RowNum: integer);
	begin
	if (RowNum < 1) or (RowNum > FRows.Count) then
		exit;

	FRows.items[RowNum - 1].free;
	invalidate;
	end;

procedure TUbViewGrid.FixLastVisibleRow;
	var
		k, rn : integer;
	begin
	rn := FRowsNumber;
	if ShowColumnHeaders then
		dec(rn); //ci sta una riga in meno a video
	if FFirstVisibleRow + rn >= FRows.Count then //ci stanno tutte a video
		FLastVisibleRow := FRows.Count - 1
	else
		begin
		FLastVisibleRow := FFirstVisibleRow + rn; //alla peggio (niente titoli);
		for k := FFirstVisibleRow + rn downto FFirstVisibleRow + 1 do //sulla prima il titolo va cmq bene
			if FRows.Items[k].FType = rtHeader then
				begin
				FLastVisibleRow := k - 1; //riga precedente ultimo title
				break;
				end;
		end;
	end;

procedure TUbViewGrid.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
  begin
  inherited;
  RowsNumber := RowsNumber; //per fissare la dimensione
  end;

//////////////////////////
///  TUbImage
//////////////////////////

constructor TUbImage.Create(AOwner: TComponent);
  begin
  inherited;
//  ControlStyle := ControlStyle +  [ csOpaque ];  
  end;

procedure TUbImage.Paint;
  begin
    inherited Canvas.Draw( 0, 0, Picture.Graphic);
  end;

procedure TUbImage.WMEraseBkgnd(var Message: TWMEraseBkgnd);
  begin
  if Transparent then
    inherited;
  end;
  

//////////////////////////
///  Register
//////////////////////////

procedure Register;
	begin
	RegisterClass(TUbColumn);
	RegisterComponents('Samples', [TUbView]);
	RegisterComponents('Samples', [TUbViewGrid]);
	RegisterComponents('Samples', [TUbImage]);
	end;

{ TUbImage }


end.
